/* Compile by "gcc -O2 -Wall -o rect_no-rt rect_no-rt.c"  */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <sys/io.h>

#define PORT 0x378
#define NSEC_PER_SEC    1000000000

static int end;

/* the struct timespec consists of nanoseconds
 * and seconds. if the nanoseconds are getting
 * bigger than 1000000000 (= 1 second) the
 * variable containing seconds has to be
 * incremented and the nanoseconds decremented
 * by 1000000000.
 */
static inline void tsnorm(struct timespec *ts)
{
   while (ts->tv_nsec >= NSEC_PER_SEC) {
      ts->tv_nsec -= NSEC_PER_SEC;
      ts->tv_sec++;
   }
}

/* increment counter and write to parallelport */
void out()
{
   static unsigned char state=0;
   outb(state++,PORT);
}

static void endme (int dummy){
  end=1;
}

int main(int argc,char** argv)
{
   struct timespec ts;
   /* default interval = 500000ns = 500us
    * cycle duration = 250us
    */
   int interval=500000;

   signal(SIGINT, endme);

   /* set permissions of parallelport */
   ioperm(PORT,1,1);

   if(argc>=2 && atoi(argv[1])>0)
      interval=atoi(argv[1]);

   printf("not using realtime, interval:i %dns\n", interval);
   
   ts.tv_sec=0;
   ts.tv_nsec=interval;
   tsnorm(&ts);
   
   while(!end){
      nanosleep(&ts, NULL);
      out();
   }

   ioperm(PORT,1,0);

   return 0;
}

