/* Compile by "gcc -O2 -Wall -lrt -o rect_rt rect_rt.c"  */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sched.h>
#include <signal.h>
#include <sys/io.h>

#define PORT 0x378
#define NSEC_PER_SEC    1000000000

static int end;

/* using clock_nanosleep of librt */
extern int clock_nanosleep(clockid_t __clock_id, int __flags,
      __const struct timespec *__req,
      struct timespec *__rem);

/* the struct timespec consists of nanoseconds
 * and seconds. if the nanoseconds are getting
 * bigger than 1000000000 (= 1 second) the
 * variable containing seconds has to be
 * incremented and the nanoseconds decremented
 * by 1000000000.
 */
static inline void tsnorm(struct timespec *ts)
{
   while (ts->tv_nsec >= NSEC_PER_SEC) {
      ts->tv_nsec -= NSEC_PER_SEC;
      ts->tv_sec++;
   }
}

/* increment counter and write to parallelport */
void lpt_out()
{
   static unsigned char state=0;
   outb(state++,PORT);
}

static void endme (int dummy){
  end=1;
}

int main(int argc,char** argv)
{
   int interval=500000;
   int priority=99;

   struct timespec ts;
   struct sched_param param;
   /* default interval = 500000ns = 500us
    * cycle duration = 250us
    */

   signal(SIGINT, endme);

   if(argc>=2 && atoi(argv[1])>=0 && atoi(argv[1])<100)
      priority = atoi(argv[1]);
 
   if(priority>0){ 
      param.sched_priority = priority;
      /* enable realtime fifo scheduling */
      if(sched_setscheduler(0, SCHED_FIFO, &param)==-1){
         perror("sched_setscheduler failed");
         exit(-1);
      }
   }

   if(argc>=3)
      interval=atoi(argv[2]);

   if (priority>0)
      printf("using realtime, priority: %d, interval: %dns\n",priority,interval);
   else
      printf("not using realtime (priority: %d), interval: %dns\n",priority,interval);

   /* set permissions of parallelport */
   ioperm(PORT,1,1);

   /* get current time */
   clock_gettime(0,&ts);
   /* start after one second */
   ts.tv_sec++;
   while(!end){
      /* wait untill next shot */
      clock_nanosleep(0, TIMER_ABSTIME, &ts, NULL);
      /* do the stuff */
      lpt_out();
      /* calculate next shot */
      ts.tv_nsec+=interval;
      tsnorm(&ts);
   }

   ioperm(PORT,1,0);
   
   return 0;
}

