/* Compile by "gcc -I/usr/xenomai/include -D_GNU_SOURCE -D_REENTRANT -D__XENO__ -lnative -L/usr/xenomai/lib -lxenomai -lpthread -lrt -Xlinker -rpath -Xlinker /usr/xenomai/lib -o rect_xenomai rect_xenomai.c" */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/mman.h>
#include <native/task.h>
#include <native/timer.h>
#include <sys/io.h>

#define PORT 0x378

RT_TASK rt_lpt_task;

/* NOTE: error handling omitted. */

void lpt_out(void *arg)
{
  static unsigned char state=0;

  /*
  * Arguments: &task (NULL=self),
  * start time,
  * period
  */
  rt_task_set_periodic(NULL, TM_NOW, (RTIME)*(int*)arg);

  while (1) {
    rt_task_wait_period(NULL);

    outb(state++,PORT);

  }
}

void catch_signal(int sig)
{
}

int main(int argc, char* argv[])
{
  int interval=500000;
  int priority=1;

  signal(SIGTERM, catch_signal);
  signal(SIGINT, catch_signal);

  if(argc>=2 && atoi(argv[1])>0 && atoi(argv[1])<100){
    priority = atoi(argv[1]);
  }

  // interval values < 20000 may cause the system to crash
  if(argc>=3 && atoi(argv[2])>=20000){
    interval=atoi(argv[2]);
  }

  printf("Using Xenomai realtime, priority: %d, interval: %dns\n",priority,interval);

  /* Set permissions for LPT access */
  ioperm(PORT,1,1);

  /* Avoids memory swapping for this program */
  mlockall(MCL_CURRENT|MCL_FUTURE);

  /*
  * Arguments: &task,
  * name,
  * stack size (0=default),
  * priority,
  * mode (FPU, start suspended, ...)
  */
  rt_task_create(&rt_lpt_task, "lpt_task", 0, priority, 0);

  /*
  * Arguments: &task,
  * task function,
  * function argument
  */
  rt_task_start(&rt_lpt_task, &lpt_out, (int*)&interval);

  pause();

  /* Set permissions for LPT access */
  ioperm(PORT,1,0);

  rt_task_delete(&rt_lpt_task);
  return 0;
}

